package org.jeecg.modules.maintenanceWork.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.convert.Convert;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.jeecg.common.util.UUIDGenerator;
import org.jeecg.modules.dynamicStaticAnalysis.mapper.AnalysisAlgorithmMapper;
import org.jeecg.modules.maintenanceWork.dto.*;
import org.jeecg.modules.maintenanceWork.entity.WorkBatch;
import org.jeecg.modules.maintenanceWork.entity.WorkBatchDetail;
import org.jeecg.modules.maintenanceWork.mapper.WorkBatchDetailMapper;
import org.jeecg.modules.maintenanceWork.mapper.WorkBatchMapper;
import org.jeecg.modules.maintenanceWork.service.IWorkBatchService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.jeecg.modules.maintenanceWork.vo.WorkBatchVO;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 维修作业管理-作业计划批次管理表 服务实现类
 * </p>
 *
 * @author hkl
 * @since 2023-08-09
 */
@Service
public class WorkBatchServiceImpl extends ServiceImpl<WorkBatchMapper, WorkBatch> implements IWorkBatchService {
    @Resource
    private WorkBatchDetailMapper workBatchDetailMapper;

    @Resource
    private AnalysisAlgorithmMapper analysisAlgorithmMapper;

    @Override
    public Page<WorkBatchVO> listRecord(Page<WorkBatchVO> pageData, WorkBatchDTO dto) {
        return this.baseMapper.listRecord(pageData, dto);
    }

    @Override
    public void saveRecord(WorkBatchSaveOrUpdateDTO dto) {
        WorkBatch workBatch = BeanUtil.copyProperties(dto, WorkBatch.class);
        workBatch.setId(UUIDGenerator.generate());
        this.save(workBatch);
    }

    @Override
    public void updateRecord(WorkBatchSaveOrUpdateDTO dto) {
        WorkBatch workBatch = BeanUtil.copyProperties(dto, WorkBatch.class);
        this.updateById(workBatch);
    }

    @Override
    public void deleteRecord(String workBatchId) {
        this.removeById(workBatchId);


        LambdaQueryWrapper<WorkBatchDetail> delWrapper = Wrappers.lambdaQuery();
        delWrapper.eq(WorkBatchDetail::getWorkBatchId, workBatchId);
        workBatchDetailMapper.delete(delWrapper);
    }

    @Override
    public void transfiniteSaveRecord(WorkBatchTransfiniteSaveDTO dto) {
        WorkBatch workBatch = BeanUtil.copyProperties(dto, WorkBatch.class);
        workBatch.setId(UUIDGenerator.generate());
        workBatch.setWorkStatus(9); // 9-已作业
        workBatch.setSource(2); // 超限页面
        this.save(workBatch);

        List<DispatchDTO> transfiniteList = dto.getDispatchList();
        for (DispatchDTO transfiniteDTO : transfiniteList) {
            WorkBatchDetail workBatchDetail = new WorkBatchDetail();
            workBatchDetail.setId(UUIDGenerator.generate());
            workBatchDetail.setWorkBatchId(workBatch.getId());
            workBatchDetail.setWorkBatchCode(workBatch.getWorkBatchCode());
            workBatchDetail.setAnalysisBatchUnitDeviceId(transfiniteDTO.getId());
            workBatchDetail.setUnitId(transfiniteDTO.getUnitId());
            workBatchDetail.setUnitCode(transfiniteDTO.getUnitCode());
            workBatchDetail.setUnitType(transfiniteDTO.getUnitType());
            workBatchDetail.setUnitTypeName(transfiniteDTO.getUnitTypeName());
            workBatchDetail.setUnitDeviceCode(transfiniteDTO.getUnitDeviceCode());
            workBatchDetail.setUnitEndMileage(transfiniteDTO.getUnitStartingMileage());
            workBatchDetail.setUnitEndMileage(transfiniteDTO.getUnitEndMileage());
            workBatchDetail.setUnitCenterMileage(transfiniteDTO.getUnitCenterMileage());
            workBatchDetail.setUnitScore(transfiniteDTO.getUnitScore());
            workBatchDetail.setUnitScoreLevel(transfiniteDTO.getUnitScoreLevel());
            workBatchDetail.setUnitScoreLevelStr(transfiniteDTO.getUnitScoreLevelStr());
            workBatchDetailMapper.insert(workBatchDetail);
        }
    }


    @Override
    public void saveDetailRecord(WorkBatchDetailDTO dto) {
        // 先删除
        LambdaQueryWrapper<WorkBatchDetail> delWrapper = Wrappers.lambdaQuery();
        delWrapper.eq(WorkBatchDetail::getWorkBatchId, dto.getWorkBatchId());
        workBatchDetailMapper.delete(delWrapper);

        List<DispatchDTO> transfiniteList = dto.getUnitDeviceList();
        for (DispatchDTO dispatchDTO : transfiniteList) {
            WorkBatchDetail workBatchDetail = new WorkBatchDetail();
            workBatchDetail.setId(UUIDGenerator.generate());
            workBatchDetail.setWorkBatchId(dto.getWorkBatchId());
            workBatchDetail.setWorkBatchCode(dto.getWorkBatchCode());
            workBatchDetail.setUnitId(dispatchDTO.getUnitId());
            workBatchDetail.setUnitCode(dispatchDTO.getUnitCode());
            workBatchDetail.setUnitType(dispatchDTO.getUnitType());
            workBatchDetail.setUnitTypeName(dispatchDTO.getUnitTypeName());
            workBatchDetail.setUnitDeviceCode(dispatchDTO.getUnitDeviceCode());
            workBatchDetail.setUnitEndMileage(dispatchDTO.getUnitStartingMileage());
            workBatchDetail.setUnitEndMileage(dispatchDTO.getUnitEndMileage());
            workBatchDetail.setUnitCenterMileage(dispatchDTO.getUnitCenterMileage());
            workBatchDetail.setUnitScore(dispatchDTO.getUnitScore());
            workBatchDetail.setUnitScoreLevel(dispatchDTO.getUnitScoreLevel());
            workBatchDetail.setUnitScoreLevelStr(dispatchDTO.getUnitScoreLevelStr());
            workBatchDetailMapper.insert(workBatchDetail);
        }
    }

    @Override
    public List<Map<String, Object>> getUnitDeviceList(UnitDeviceDTO dto) {
        List<Map<String, Object>> resultList = new ArrayList<>();
        List<Map<String, Object>> unitDeviceList = analysisAlgorithmMapper.getUnitDevice(dto.getLightRailId(), dto.getLineAliasId());

        for (Map<String, Object> unitDevice : unitDeviceList) {
            BigDecimal startMileage = Convert.toBigDecimal(unitDevice.get("startMileage"));
            BigDecimal endMileage = Convert.toBigDecimal(unitDevice.get("endMileage"));
            BigDecimal centerMileage = startMileage.add(endMileage).divide(new BigDecimal("2"), 3, RoundingMode.HALF_UP);
            if (dto.getStartMileage().compareTo(centerMileage) <= 0 && dto.getEndMileage().compareTo(centerMileage) > 0) {
                resultList.add(unitDevice);
            }
        }
        return resultList;
    }

}
